package com.yundian.flutter.alive.yundian_alive

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.yundian.baseui.utils.AesUtils
import com.yundian.sdk.android.alive.api.AliveManager
import com.yundian.sdk.android.alive.ui.api.AliveDetectedResultListener
import com.yundian.sdk.android.alive.ui.api.AliveUiEngine
import com.yundian.sdk.android.alive.ui.api.AliveUiManager
import com.yundian.sdk.android.ocr.ui.api.OcrDetectedResultListener
import com.yundian.sdk.android.ocr.ui.api.OcrUiEngine
import com.yundian.sdk.android.ocr.ui.api.OcrUiManager

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** YundianAlivePlugin */
class YundianAlivePlugin : FlutterPlugin, MethodCallHandler {
    private var channel: MethodChannel? = null
    private var aliveResultListener: EventChannel? = null
    private var ocrResultListener: EventChannel? = null
    private lateinit var context: Context
    private var aliveEventSink: EventChannel.EventSink? = null
    private var ocrEventSink: EventChannel.EventSink? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "yundian_alive")
        channel?.setMethodCallHandler(this)

        aliveResultListener = EventChannel(flutterPluginBinding.binaryMessenger, "yundian_alive.event.alive.result_listener")
        aliveResultListener?.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(p0: Any?, eventSink: EventChannel.EventSink?) {
                aliveEventSink = eventSink
            }

            override fun onCancel(p0: Any?) {
                aliveEventSink?.endOfStream()
            }
        })
        ocrResultListener = EventChannel(flutterPluginBinding.binaryMessenger, "yundian_alive.event.ocr.result_listener")
        ocrResultListener?.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(p0: Any?, eventSink: EventChannel.EventSink?) {
                ocrEventSink = eventSink
            }

            override fun onCancel(p0: Any?) {
                ocrEventSink?.endOfStream()
            }
        })
    }

    private fun getLayoutIdByName(layoutName: String): Int {
        return context.getResources().getIdentifier(layoutName, "layout", context.getPackageName());
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "getPlatformVersion" -> result.success(" alive Android ${android.os.Build.VERSION.RELEASE}")
            "alive_encrypt_aes" -> {
                var info = call.argument("info") as? String
                var appSecret = call.argument("appSecret") as? String
                result.success(AesUtils.encryptAes(info, appSecret))
            }
            "alive_init" -> {
                var isDebug = call.argument("isDebug") as? Boolean ?: false
                var appKey = call.argument("appKey") as? String
                var mchInfo = call.argument("mchInfo") as? String
                var riskInfo = call.argument("riskInfo") as? String
                AliveUiManager.getInstance().init(context, isDebug, appKey, mchInfo, riskInfo)
                result.success("alive_init success")
            }
            "aliveRelease" -> {
                AliveManager.getInstance().release()
                AliveUiEngine.getInstance().release()
                result.success("alive_release success")
            }
            "setLayoutAliveDetectedId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { AliveUiEngine.getInstance().setLayoutAliveDetectedId(getLayoutIdByName(it)) }
                result.success("alive_layoutAliveDetectedId success")
            }
            "setLayoutAliveDetectedFailId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { AliveUiEngine.getInstance().setLayoutAliveDetectedFailId(getLayoutIdByName(it)) }
                result.success("alive_layoutDetectedFailId success")
            }
            "setLayoutAliveDetectedPreId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { AliveUiEngine.getInstance().setLayoutAliveDetectedPreId(getLayoutIdByName(it)) }
                result.success("alive_layoutDetectedPreId success")
            }
            "setLayoutAliveDetectedSuccessId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { AliveUiEngine.getInstance().setLayoutAliveDetectedSuccessId(getLayoutIdByName(it)) }
                result.success("alive_layoutDetectedSuccessId success")
            }
            "setTimeout" -> {
                var timeout = call.argument("timeout") as? Int
                timeout?.let { AliveUiEngine.getInstance().setTimeout(it) }
                result.success("alive_timeout success")
            }
            "setActionSize" -> {
                var actionSize = call.argument("actionSize") as? Int
                actionSize?.let { AliveUiEngine.getInstance().setActionSize(it) }
                result.success("alive_actionSize success")
            }
            "setSecurityLevel" -> {
                var securityLevel = call.argument("securityLevel") as? Int
                securityLevel?.let { AliveUiEngine.getInstance().setSecurityLevel(it) }
                result.success("alive_securityLevel success")
            }
            "setActionSecond" -> {
                var actionSecond = call.argument("actionSecond") as? Int
                actionSecond?.let { AliveUiEngine.getInstance().setActionSecond(it) }
                result.success("alive_actionSecond success")
            }
            "alive_start_detect" -> {
                var hasResultPage = call.argument("hasResultPage") as? Boolean ?: false
                var authToken = call.argument("authToken") as? String
                var distance = call.argument("distance") as? Float ?: 3f
                Log.d(">>>>>>>>>>", "" + distance)
                AliveUiManager.getInstance().startActivity(context, authToken, distance, hasResultPage, object : AliveDetectedResultListener {
                    override fun success(score: Double, featureImageId: String) {
                        aliveEventSink?.success(AliveResult(0, "success", score, featureImageId).toResult())
                    }

                    override fun cancel(msg: String?) {
                        aliveEventSink?.success(AliveResult(1, msg, 0.0, null).toResult())
                    }

                    override fun fail(msg: String?) {
                        aliveEventSink?.success(AliveResult(2, msg, 0.0, null).toResult())
                    }
                });
                result.success("alive_start_detect success")
            }

            //*************************************OCR 部分

            "ocr_init" -> {
                var isDebug = call.argument("isDebug") as? Boolean ?: false
                var appKey = call.argument("appKey") as? String
                var mchInfo = call.argument("mchInfo") as? String
                var riskInfo = call.argument("riskInfo") as? String
                OcrUiManager.getInstance().init(context, isDebug, appKey, mchInfo, riskInfo)
                result.success("alive_init success")
            }
            "ocrRelease" -> {
                OcrUiEngine.getInstance().release()
                result.success("setLayoutIdCardOcrId success")
            }
            "setLayoutIdCardOcrId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutIdCardOcrId(getLayoutIdByName(it)) }
                result.success("setLayoutIdCardOcrId success")
            }
            "setLayoutIdCardOcrSingleId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutIdCardOcrSingleId(getLayoutIdByName(it)) }
                result.success("setLayoutIdCardOcrSingleId success")
            }
            "setLayoutCarOcrId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutCarOcrId(getLayoutIdByName(it)) }
                result.success("setLayoutCarOcrId success")
            }
            "setLayoutIdCardResultId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutIdCardResultId(getLayoutIdByName(it)) }
                result.success("setLayoutIdCardResultId success")
            }
            "setLayoutRealPeopleVerifyId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutRealPeopleVerifyId(getLayoutIdByName(it)) }
                result.success("setLayoutRealPeopleVerifyId success")
            }
            "setLayoutRealPeopleVerifySuccessId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutRealPeopleVerifySuccessId(getLayoutIdByName(it)) }
                result.success("setLayoutRealPeopleVerifySuccessId success")
            }
            "setLayoutRealPeopleVerifyFailedId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutRealPeopleVerifyFailedId(getLayoutIdByName(it)) }
                result.success("setLayoutRealPeopleVerifyFailedId success")
            }
            "setLayoutTakePhoneId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutTakePhoneId(getLayoutIdByName(it)) }
                result.success("setLayoutTakePhoneId success")
            }
            "setLayoutUploadIdCardId" -> {
                var layoutId = call.argument("layoutIdName") as? String
                layoutId?.let { OcrUiEngine.getInstance().setLayoutUploadIdCardId(getLayoutIdByName(it)) }
                result.success("setLayoutUploadIdCardId success")
            }
            "setUploadTvBackTextColor" -> {
                var color = call.argument("color") as? String
                color?.let { OcrUiEngine.getInstance().setUploadTvBackTextColor(it) }
                result.success("setUploadTvBackTextColor success")
            }
            "setUploadIvBackRes" -> {
                var uploadIvBackRes = call.argument("uploadIvBackRes") as? Int
                uploadIvBackRes?.let { OcrUiEngine.getInstance().setUploadIvBackRes(uploadIvBackRes) }
                result.success("setUploadIvBackRes success")
            }
            "ocr_start_detect" -> {
                var hasResultPage = call.argument("hasResultPage") as? Boolean ?: false
                var authToken = call.argument("authToken") as? String
                OcrUiManager.getInstance().startActivity(context, authToken, hasResultPage, object : OcrDetectedResultListener {

                    override fun success(json: String?) {
                        ocrEventSink?.success(OcrResult(0, "success", json).toResult())
                    }

                    override fun cancel(msg: String?) {
                        ocrEventSink?.success(OcrResult(1, msg, null).toResult())
                    }

                    override fun fail(msg: String?) {
                        ocrEventSink?.success(OcrResult(2, msg, null).toResult())
                    }
                });
                result.success("alive_start_detect success")
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel?.setMethodCallHandler(null)
    }
}
