import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'yundian_alive_platform_interface.dart';
import 'yundian_ocr_platform_interface.dart';

/// An implementation of [YundianAlivePlatform] that uses method channels.
class MethodChannelYundianOcr extends YundianOcrPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('yundian_alive');

  final ocrResultListener = const EventChannel('yundian_alive.event.ocr.result_listener');

  @override
  Future<String?> ocrInit(bool isDebug, String appKey, String mchInfo, String riskInfo) async {
    var parameters = {"isDebug": isDebug, "appKey": appKey, "mchInfo": mchInfo, "riskInfo": riskInfo};
    final version = await methodChannel.invokeMethod<String>('ocr_init', Map.from(parameters));
    return version;
  }

  @override
  Future<String?> ocrStartDetect(String authToken, bool isRealPeopleVerify) async {
    var parameters = {"authToken": authToken, "isRealPeopleVerify": isRealPeopleVerify};
    final version = await methodChannel.invokeMethod<String>('ocr_start_detect', Map.from(parameters));
    return version;
  }

  @override
  onOcrResultListener(EventHandler onEvent, EventHandler onError) {
    ocrResultListener.receiveBroadcastStream().listen(onEvent, onError: onError);
  }

  @override
  Future<String?> ocrRelease() async{
    final version = await methodChannel.invokeMethod<String>('ocrRelease');
    return version;
  }

  @override
  Future<String?> setLayoutIdCardOcrId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutIdCardOcrId', Map.from(parameters));
    return version;
  }

  @override
  Future<String?> setLayoutIdCardOcrSingleId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutIdCardOcrSingleId', Map.from(parameters));
    return version;
  }

  @override
  Future<String?> setLayoutCarOcrId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutCarOcrId', Map.from(parameters));
    return version;
  }

  @override
  Future<String?> setLayoutIdCardResultId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutIdCardResultId', Map.from(parameters));
    return version;
  }

  @override
  Future<String?> setLayoutRealPeopleVerifyId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutRealPeopleVerifyId', Map.from(parameters));
    return version;
  }

  @override
  Future<String?> setLayoutRealPeopleVerifySuccessId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutRealPeopleVerifySuccessId', Map.from(parameters));
    return version;
  }
  @override
  Future<String?> setLayoutRealPeopleVerifyFailedId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutRealPeopleVerifyFailedId', Map.from(parameters));
    return version;
  }
  @override
  Future<String?> setLayoutTakePhoneId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutTakePhoneId', Map.from(parameters));
    return version;
  }
  @override
  Future<String?> setLayoutUploadIdCardId(String layoutIdName) async{
    var parameters = {"layoutIdName": layoutIdName};
    final version = await methodChannel.invokeMethod<String>('setLayoutUploadIdCardId', Map.from(parameters));
    return version;
  }

  @override
  Future<String?> setUploadTvBackTextColor(String color) async{
    var parameters = {"color": color};
    final version = await methodChannel.invokeMethod<String>('setUploadTvBackTextColor', Map.from(parameters));
    return version;
  }


  @override
  Future<String?> setUploadIvBackRes(int uploadIvBackRes) async{
    var parameters = {"uploadIvBackRes": uploadIvBackRes};
    final version = await methodChannel.invokeMethod<String>('setUploadIvBackRes', Map.from(parameters));
    return version;
  }

}

typedef EventHandler = void Function(dynamic event);
